@@ -1,22 +1,22 @@
+mod build;
+mod pack;
+mod pipeline;
+
+use crate::{
+    build::{build_project, BuildProfile},
+    pack::pack_assets_and_write_to_file,
+    pipeline::{Pipeline, PipelineCommand},
+};
 use cargo_metadata::MetadataCommand;
 use clap::{App, Arg, SubCommand};
 use dirs::home_dir;
 use hotwatch::{Event, Hotwatch};
-use oxygengine_build_tools::{
-    atlas::pack_sprites_and_write_to_files,
-    build::{build_project, BuildProfile},
-    pack::pack_assets_and_write_to_file,
-    pipeline::{AtlasPhase, CopyPhase, PackPhase, Pipeline, TiledPhase},
-    tiled::build_map_and_write_to_file,
-};
 use oxygengine_ignite_types::IgniteTypeDefinition;
 use serde::Deserialize;
 use std::{
     collections::{HashMap, HashSet},
     env::{current_dir, current_exe, set_current_dir, vars},
-    fs::{
-        copy, create_dir_all, read, read_dir, read_to_string, remove_dir_all, remove_file, write,
-    },
+    fs::{copy, create_dir_all, read_dir, read_to_string, remove_dir_all, remove_file, write},
     io::{Error, ErrorKind, Result},
     path::{Path, PathBuf},
     process::{Command, Stdio},
@@ -247,7 +247,7 @@ async fn main() -> Result<()> {
                         .help(&format!("Project preset ({})", presets_list.join(", ")))
                         .takes_value(false)
                         .required(false)
-                        .default_value("web-composite-game"),
+                        .default_value("web-ha-game"),
                 )
                 .arg(
                     Arg::with_name("dont-build")
@@ -256,14 +256,6 @@ async fn main() -> Result<()> {
                         .takes_value(false)
                         .required(false),
                 )
-                .arg(
-                    Arg::with_name("quiet")
-                        .short("q")
-                        .long("quiet")
-                        .help("Don't show progress information")
-                        .takes_value(false)
-                        .required(false),
-                ),
         )
         .subcommand(
             SubCommand::with_name("pack")
@@ -286,148 +278,6 @@ async fn main() -> Result<()> {
                         .takes_value(true)
                         .required(true),
                 )
-                .arg(
-                    Arg::with_name("quiet")
-                        .short("q")
-                        .long("quiet")
-                        .help("Don't show progress information")
-                        .takes_value(false)
-                        .required(false),
-                ),
-        )
-        .subcommand(
-            SubCommand::with_name("atlas")
-                .about("Pack images into sprite sheet (texture atlas)")
-                .arg(
-                    Arg::with_name("input")
-                        .short("i")
-                        .long("input")
-                        .value_name("PATH")
-                        .help("Image file or folder containing images")
-                        .takes_value(true)
-                        .required(true)
-                        .multiple(true),
-                )
-                .arg(
-                    Arg::with_name("output-image")
-                        .short("o")
-                        .long("output-image")
-                        .value_name("PATH")
-                        .help("Image output file")
-                        .takes_value(true)
-                        .required(true),
-                )
-                .arg(
-                    Arg::with_name("output-atlas")
-                        .short("a")
-                        .long("output-atlas")
-                        .value_name("PATH")
-                        .help("Spritesheet output file")
-                        .takes_value(true)
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("max-width")
-                        .short("w")
-                        .long("max-width")
-                        .value_name("NUMBER")
-                        .help("Maximum atlas image width")
-                        .takes_value(true)
-                        .default_value("2048")
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("max-height")
-                        .short("h")
-                        .long("max-height")
-                        .value_name("NUMBER")
-                        .help("Maximum atlas image height")
-                        .takes_value(true)
-                        .default_value("2048")
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("padding")
-                        .short("p")
-                        .long("padding")
-                        .value_name("NUMBER")
-                        .help("Padding between atlas images")
-                        .takes_value(true)
-                        .default_value("2")
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("pretty")
-                        .short("r")
-                        .long("pretty")
-                        .help("Produce pretty human-readable JSON")
-                        .takes_value(false)
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("full-names")
-                        .short("f")
-                        .long("full-names")
-                        .help("Give full name (with parent folders) to frame ID")
-                        .takes_value(false)
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("quiet")
-                        .short("q")
-                        .long("quiet")
-                        .help("Don't show progress information")
-                        .takes_value(false)
-                        .required(false),
-                ),
-        )
-        .subcommand(
-            SubCommand::with_name("tiled")
-                .about("Build Tiled JSON file into set of engine map related files")
-                .arg(
-                    Arg::with_name("input")
-                        .short("i")
-                        .long("input")
-                        .value_name("PATH")
-                        .help("Tiled JSON file")
-                        .takes_value(true)
-                        .required(true),
-                )
-                .arg(
-                    Arg::with_name("output")
-                        .short("o")
-                        .long("output")
-                        .value_name("PATH")
-                        .help("Map binary file")
-                        .takes_value(true)
-                        .required(true),
-                )
-                .arg(
-                    Arg::with_name("spritesheet")
-                        .short("s")
-                        .long("spritesheet")
-                        .value_name("PATH")
-                        .help("Sprite sheet (texture atlas) JSON file")
-                        .takes_value(true)
-                        .required(true)
-                        .multiple(true),
-                )
-                .arg(
-                    Arg::with_name("full-names")
-                        .short("f")
-                        .long("full-names")
-                        .help("Give full name (with parent folders) to sprites used")
-                        .takes_value(false)
-                        .required(false),
-                )
-                .arg(
-                    Arg::with_name("quiet")
-                        .short("q")
-                        .long("quiet")
-                        .help("Don't show progress information")
-                        .takes_value(false)
-                        .required(false),
-                ),
         )
         .subcommand(
             SubCommand::with_name("pipeline")
@@ -450,14 +300,6 @@ async fn main() -> Result<()> {
                         .takes_value(false)
                         .required(false),
                 )
-                .arg(
-                    Arg::with_name("dry-run")
-                        .short("d")
-                        .long("dry-run")
-                        .help("Perform dry run (don't execute pipeline, just show its flow)")
-                        .takes_value(false)
-                        .required(false),
-                ),
         )
         .subcommand(
             SubCommand::with_name("build")
@@ -535,6 +377,16 @@ async fn main() -> Result<()> {
         .subcommand(
             SubCommand::with_name("live")
                 .about("Listen for changes in binary and asset sources and rebuild them when they change")
+                .arg(
+                    Arg::with_name("profile")
+                        .short("r")
+                        .long("profile")
+                        .value_name("PROFILE")
+                        .help("Project build profile. Possible values: debug, release")
+                        .takes_value(true)
+                        .required(false)
+                        .default_value("debug"),
+                )
                 .arg(
                     Arg::with_name("binaries")
                         .short("b")
@@ -676,7 +528,6 @@ async fn main() -> Result<()> {
         let destination = matches.value_of("destination");
         let preset = matches.value_of("preset").unwrap();
         let dont_build = matches.is_present("dont-build");
-        let quiet = matches.is_present("quiet");
         let preset_path = presets_path.join(preset);
         if !preset_path.exists() {
             return Err(Error::new(
@@ -716,33 +567,27 @@ async fn main() -> Result<()> {
             None
         };
 
-        if !quiet {
-            println!("Make project: {:?}", &destination_path);
-            if let Some(preset_manifest) = &preset_manifest {
-                preset_manifest.print_note(ActionType::PreCreate);
-            }
-            println!("* Prepare project structure...");
+        println!("Make project: {:?}", &destination_path);
+        if let Some(preset_manifest) = &preset_manifest {
+            preset_manifest.print_note(ActionType::PreCreate);
         }
+        println!("* Prepare project structure...");
         if let Some(preset_manifest) = &preset_manifest {
             preset_manifest.execute_script(ActionType::PreCreate, &destination_path)?;
         }
-        copy_dir(&preset_path, &destination_path, &id)?;
+        copy_dir(&preset_path, &destination_path, id)?;
         if let Some(preset_manifest) = &preset_manifest {
             preset_manifest.execute_script(ActionType::PostCreate, &destination_path)?;
         }
-        if !quiet {
-            println!("  Done!");
-            if let Some(preset_manifest) = &preset_manifest {
-                preset_manifest.print_note(ActionType::PostCreate);
-            }
+        println!("Done!");
+        if let Some(preset_manifest) = &preset_manifest {
+            preset_manifest.print_note(ActionType::PostCreate);
         }
         if !dont_build {
-            if !quiet {
-                if let Some(preset_manifest) = &preset_manifest {
-                    preset_manifest.print_note(ActionType::PreBuild);
-                }
-                println!("* Build rust project...");
+            if let Some(preset_manifest) = &preset_manifest {
+                preset_manifest.print_note(ActionType::PreBuild);
             }
+            println!("* Build rust project...");
             if let Some(preset_manifest) = &preset_manifest {
                 preset_manifest.execute_script(ActionType::PreBuild, &destination_path)?;
             }
@@ -753,97 +598,49 @@ async fn main() -> Result<()> {
             if let Some(preset_manifest) = &preset_manifest {
                 preset_manifest.execute_script(ActionType::PostBuild, &destination_path)?;
             }
-            if !quiet {
-                println!("  Done!");
-                if let Some(preset_manifest) = &preset_manifest {
-                    preset_manifest.print_note(ActionType::PostBuild);
-                }
+            println!("Done!");
+            if let Some(preset_manifest) = &preset_manifest {
+                preset_manifest.print_note(ActionType::PostBuild);
             }
         }
     } else if let Some(matches) = matches.subcommand_matches("pack") {
         let input = matches.values_of("input").unwrap().collect::<Vec<_>>();
         let output = matches.value_of("output").unwrap();
-        let quiet = matches.is_present("quiet");
-        pack_assets_and_write_to_file(&input, output, quiet)?;
-    } else if let Some(matches) = matches.subcommand_matches("atlas") {
-        let input = matches.values_of("input").unwrap().collect::<Vec<_>>();
-        let output_image = matches.value_of("output-image").unwrap();
-        let output_atlas = &if let Some(path) = matches.value_of("output-atlas") {
-            path.to_owned()
-        } else {
-            Path::new(output_image)
-                .with_extension("json")
-                .to_str()
-                .unwrap()
-                .to_owned()
-        };
-        let max_width = matches.value_of("max-width").unwrap().parse().unwrap();
-        let max_height = matches.value_of("max-height").unwrap().parse().unwrap();
-        let padding = matches.value_of("padding").unwrap().parse().unwrap();
-        let pretty = matches.is_present("pretty");
-        let full_names = matches.is_present("full-names");
-        let quiet = matches.is_present("quiet");
-        pack_sprites_and_write_to_files(
-            &input,
-            output_image,
-            output_atlas,
-            max_width,
-            max_height,
-            padding,
-            pretty,
-            full_names,
-            quiet,
-        )?;
-    } else if let Some(matches) = matches.subcommand_matches("tiled") {
-        let input = matches.value_of("input").unwrap();
-        let output = matches.value_of("output").unwrap();
-        let spritesheets = matches
-            .values_of("spritesheet")
-            .unwrap()
-            .collect::<Vec<_>>();
-        let full_names = matches.is_present("full-names");
-        let quiet = matches.is_present("quiet");
-        build_map_and_write_to_file(input, output, &spritesheets, full_names, quiet)?;
+        pack_assets_and_write_to_file(&input, output)?;
     } else if let Some(matches) = matches.subcommand_matches("pipeline") {
         let config = matches.value_of("config").unwrap();
         let mut config = Path::new(&config).to_owned();
-        let dry_run = matches.is_present("dry-run");
         let template = matches.is_present("template");
         if template {
-            let pipeline = Pipeline::default()
-                .source("static")
-                .destination("static")
-                .pipeline(
-                    Pipeline::default()
-                        .destination("assets-generated")
-                        .clear_destination(true),
-                )
-                .pipeline(
-                    Pipeline::default()
-                        .source("assets-source")
-                        .destination("assets-generated")
-                        .copy(CopyPhase::default().from("assets.txt"))
-                        .atlas(
-                            AtlasPhase::default()
-                                .path("images")
-                                .output_image("sprites.png")
-                                .output_atlas("sprites.json")
-                                .pretty(true),
-                        ),
-                )
-                .pipeline(
-                    Pipeline::default().destination("assets-generated").tiled(
-                        TiledPhase::default()
-                            .input("assets-source/maps/map.json")
-                            .spritesheet("assets-generated/sprites.0.json")
-                            .output("map.map"),
-                    ),
-                )
-                .pack(
-                    PackPhase::default()
-                        .path("assets-generated")
-                        .output("assets.pack"),
-                );
+            let pipeline = Pipeline {
+                source: "static".into(),
+                destination: "static".into(),
+                commands: vec![
+                    PipelineCommand::Pipeline(Pipeline {
+                        disabled: false,
+                        destination: "assets-generated".into(),
+                        clear_destination: true,
+                        ..Default::default()
+                    }),
+                    PipelineCommand::Pipeline(Pipeline {
+                        disabled: false,
+                        source: "assets-source".into(),
+                        destination: "assets-generated".into(),
+                        commands: vec![PipelineCommand::Copy {
+                            disabled: false,
+                            from: vec!["assets.txt".into()],
+                            to: "".into(),
+                        }],
+                        ..Default::default()
+                    }),
+                    PipelineCommand::Pack {
+                        disabled: false,
+                        paths: vec!["assets-generated".into()],
+                        output: "assets.pack".into(),
+                    },
+                ],
+                ..Default::default()
+            };
             let contents = match serde_json::to_string_pretty(&pipeline) {
                 Ok(contents) => contents,
                 Err(error) => {
@@ -856,26 +653,17 @@ async fn main() -> Result<()> {
                     ))
                 }
             };
-            if dry_run {
-                println!("Pipeline: {}", contents);
-            } else {
-                write(config, &contents)?;
-            }
+            write(config, &contents)?;
         } else if config.exists() {
             let contents = read_to_string(&config)?;
             match serde_json::from_str::<Pipeline>(&contents) {
                 Ok(pipeline) => {
-                    if dry_run {
-                        pipeline.dry_run();
-                    } else {
-                        set_current_dir(if config.is_file() {
-                            config.pop();
-                            config
-                        } else {
-                            config
-                        })?;
-                        pipeline.execute()?;
+                    verify_used_plugins(&pipeline);
+                    if config.is_file() {
+                        config.pop();
                     }
+                    set_current_dir(config)?;
+                    pipeline.execute()?;
                 }
                 Err(error) => println!(
                     "Could not parse pipeline JSON config: {:?}. Error: {:?}",
@@ -913,6 +701,7 @@ async fn main() -> Result<()> {
         }
         serve_files(port, binaries, assets).await;
     } else if let Some(matches) = matches.subcommand_matches("live") {
+        let profile = matches.value_of("profile").unwrap().to_owned();
         let binaries = matches
             .values_of("binaries")
             .map(|v| v.collect::<Vec<_>>())
@@ -966,6 +755,8 @@ async fn main() -> Result<()> {
             while build_receiver.recv().is_ok() {
                 if let Err(error) = Command::new(&exe)
                     .arg("build")
+                    .arg("--profile")
+                    .arg(&profile)
                     .arg("--crate_dir")
                     .arg(&crate_dir)
                     .status()
@@ -1078,17 +869,6 @@ async fn main() -> Result<()> {
                             });
                             let definition = {
                                 match extension {
-                                    "yaml" => {
-                                        let contents = read_to_string(path.to_owned())
-                                            .unwrap_or_else(|_| {
-                                                panic!(
-                                                    "Could not read ignite type file: {:?}",
-                                                    path
-                                                )
-                                            });
-                                        serde_yaml::from_str::<IgniteTypeDefinition>(&contents)
-                                            .expect("Could not parse YAML type definition file")
-                                    }
                                     "json" => {
                                         let contents = read_to_string(path.to_owned())
                                             .unwrap_or_else(|_| {
@@ -1100,24 +880,6 @@ async fn main() -> Result<()> {
                                         serde_json::from_str::<IgniteTypeDefinition>(&contents)
                                             .expect("Could not parse YAML type definition file")
                                     }
-                                    "ron" => {
-                                        let contents = read_to_string(path.to_owned())
-                                            .unwrap_or_else(|_| {
-                                                panic!(
-                                                    "Could not read ignite type file: {:?}",
-                                                    path
-                                                )
-                                            });
-                                        ron::from_str::<IgniteTypeDefinition>(&contents)
-                                            .expect("Could not parse YAML type definition file")
-                                    }
-                                    "bin" => {
-                                        let contents = read(path.to_owned()).unwrap_or_else(|_| {
-                                            panic!("Could not read ignite type file: {:?}", path)
-                                        });
-                                        bincode::deserialize::<IgniteTypeDefinition>(&contents)
-                                            .expect("Could not parse YAML type definition file")
-                                    }
                                     extension => panic!("Unsupported file type: {:?}", extension),
                                 }
                             };
@@ -1198,3 +960,23 @@ fn copy_dir(from: &Path, to: &Path, id: &str) -> Result<()> {
     }
     Ok(())
 }
+
+fn verify_used_plugins(pipeline: &Pipeline) {
+    for command in &pipeline.commands {
+        match command {
+            PipelineCommand::Plugin { name, .. } => {
+                if which::which(name).is_err() {
+                    let plugin = name.rfind('-').map(|index| &name[0..index]).unwrap_or(name);
+                    let plugin = format!("{}-tools", plugin);
+                    println!(
+                        "Plugin not found: {}. Trying to install it: {}",
+                        name, plugin
+                    );
+                    let _ = Command::new("cargo").arg("install").arg(plugin).status();
+                }
+            }
+            PipelineCommand::Pipeline(pipeline) => verify_used_plugins(pipeline),
+            _ => {}
+        }
+    }
+}
